package ca.recrown.islandsurvivalcraft.islandbedrockmetadata;

import java.util.List;

import org.bukkit.World;
import org.bukkit.metadata.MetadataValue;
import org.bukkit.plugin.Plugin;

public class IslandBedrockMetadataHelper {
    World world;
    Plugin plugin;

    public IslandBedrockMetadataHelper(Plugin owningPlugin, World world) {
        this.world = world;
        this.plugin = owningPlugin;
    }

    public void setIslandBedrockMetadata(int worldX, int worldZ, IslandFixedMetadata metadata) {
        world.getBlockAt(worldX, 0, worldZ).setMetadata(metadata.getIslandMetadataType().toString(), metadata);
    }

    public void setIslandBedrockMetadataPack(int worldX, int worldZ, IslandMetadataPack pack) {
        setIslandBedrockMetadata(worldX, worldZ, pack.mainBiomeMetadata);
        setIslandBedrockMetadata(worldX, worldZ, pack.shoreBiomeMetadata);
        setIslandBedrockMetadata(worldX, worldZ, pack.temperatureMetadata);
        setIslandBedrockMetadata(worldX, worldZ, pack.islandTIUDMetadata);
        setIslandBedrockMetadata(worldX, worldZ, pack.ownerMetadata);
    }

    /**
     * Gets the metadata stored in the bedrock at the given coordinates.
     * @param worldX The x coordinate of the world.
     * @param worldZ The z coordinate of the world.
     * @param type the type of island metadata to retrieve.
     * @return the island bedrock metadata at that location.
     */
    public IslandFixedMetadata getIslandBedrockMetadata(int worldX, int worldZ, IslandMetadataType type) {
        List<MetadataValue> potential = world.getBlockAt(worldX, 0, worldZ).getMetadata(type.toString());
        for (MetadataValue metadataValue : potential) {
            if (metadataValue.getOwningPlugin().equals(plugin)) {
                return (IslandFixedMetadata) metadataValue;
            }
        }
        throw new IllegalArgumentException("No metadata was found for given type.");
    }
}