package ca.recrown.islandsurvivalcraft.pathfinding;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInstance;
import org.junit.jupiter.api.TestInstance.Lifecycle;


/**
 * Unit test for simple App.
 */
@TestInstance(Lifecycle.PER_CLASS)
public class DepthFirstSearchTest {

    private class Validator implements CoordinateValidatable {
        private boolean[][] map;

        public Validator(boolean[][] map) {
            this.map = map;
        }

        @Override
        public boolean validate(int x, int y) {
            try {
                return map[y][x];
            } catch (IndexOutOfBoundsException e) {
                return false;
            }
        }
    }

    private boolean[][] mapA = new boolean[][] {
        {true, true},
        {true, false},
        {true, true}
    };

    private boolean[][] mapB = new boolean[][] {
        {true, true, true, true},
        {true, false, false, false},
        {true, true, true, true}
    };

    private boolean[][] mapC = new boolean[][] {
        {true, true, true, true},
        {false, false, false, false},
        {true, true, true, true}
    };

    private boolean[][] mapD = new boolean[][] {
        {true, true, true, true},
        {false, false, false, true},
        {true, true, true, true},
        {true, false, false, false},
    };

    private boolean[][] mapE = new boolean[][] {
        {true, true, true, true},
        {false, false, false, false},
        {true, true, true, true},
        {true, false, false, false},
    };

    @BeforeAll
    protected void setUp() {
    }

    @Test
    public void testDFSMapAValid()
    {
        DepthFirstSearch dfs = new DepthFirstSearch(new Validator(mapA));
        dfs.Setup(1, 0, 1, 2);
        assertTrue(dfs.buildTree());
    }

    @Test
    public void testDFSMapBValid()
    {
        DepthFirstSearch dfs = new DepthFirstSearch(new Validator(mapB));
        dfs.Setup(3, 2, 3, 0);
        assertTrue(dfs.buildTree());
    }

    @Test
    public void testDFSMapDValid()
    {
        DepthFirstSearch dfs = new DepthFirstSearch(new Validator(mapD));
        dfs.Setup(0, 3, 0, 0);
        assertTrue(dfs.buildTree());
    }

    @Test
    public void testDFSMapCInvalid()
    {
        DepthFirstSearch dfs = new DepthFirstSearch(new Validator(mapC));
        dfs.Setup(3, 2, 3, 0);
        assertFalse(dfs.buildTree());
    }

    @Test
    public void testDFSMapEInvalid()
    {
        DepthFirstSearch dfs = new DepthFirstSearch(new Validator(mapE));
        dfs.Setup(3, 2, 3, 0);
        assertFalse(dfs.buildTree());
    }
}
